"""Tests for utils."""

import unittest

from utils import Log, Tweet, Twitter

log = Log('test_twitter')

TEST_QUOTE = '\n'.join(
    [
        'Data is a precious thing',
        'and will last longer',
        'than the systems themselves.',
        '- Tim Berners-Lee',
    ]
)


SKIP_API_CREDENTIALS = 'Needs Twitter API credentials'


class TestActionerMixin(unittest.TestCase):
    @unittest.skip(SKIP_API_CREDENTIALS)
    def test_send_tweet(self):
        twitter = Twitter.from_environ_vars()
        tweet = Tweet(
            TEST_QUOTE,
        )
        response = twitter.send(tweet)
        self.assertIsNotNone(response)

    @unittest.skip(SKIP_API_CREDENTIALS)
    def test_send_tweet_with_media(self):
        twitter = Twitter.from_environ_vars()
        tweet = Tweet(
            TEST_QUOTE,
            ['tests/types_of_data.png'],
        )
        response = twitter.send(tweet)
        self.assertIsNotNone(response)

    @unittest.skip(SKIP_API_CREDENTIALS)
    def test_send_tweets_in_thread(self):
        twitter = Twitter.from_environ_vars()
        previous_tweet_id = None
        for i in range(0, 4):
            tweet = Tweet(
                f'{i}/ ' + TEST_QUOTE,
                ['tests/types_of_data.png'],
                previous_tweet_id,
            )
            response = twitter.send(tweet)
            self.assertIsNotNone(response)
            previous_tweet_id = response.id
            log.debug(previous_tweet_id)

    @unittest.skip(SKIP_API_CREDENTIALS)
    def test_update_profile_description(self):
        twitter = Twitter.from_environ_vars()
        twitter.update_profile_description()

    @unittest.skip(SKIP_API_CREDENTIALS)
    def test_update_profile_image(self):
        twitter = Twitter.from_environ_vars()
        twitter.update_profile_image('tests/lanka_data_logo.png')

    @unittest.skip(SKIP_API_CREDENTIALS)
    def test_update_banner_image(self):
        twitter = Twitter.from_environ_vars()
        twitter.update_banner_image('tests/banner.png')


if __name__ == '__main__':
    unittest.main()



import os

from utils.file.File import File
from utils.file.FileOrDirectory import FileOrDirectory


class Directory(FileOrDirectory):
    @staticmethod
    def isIgnore(path: str):
        for k in ['__pycache__', '.git']:
            if k in path:
                return True
        return False

    def mkdir(self):
        os.mkdir(self.path)

    @property
    def children(self):
        _children = []
        for name in os.listdir(self.path):
            path = os.path.join(self.path, name)
            if Directory.isIgnore(path):
                continue

            if os.path.isdir(path):
                _children.append(Directory(path))
            else:
                _children.append(File(path))
        return sorted(_children, key=lambda x: x.name)



import os
import tempfile
import unittest
from urllib.parse import urljoin

from utils import CSVFile, File, JSONFile, TSVFile, XSVFile
from utils.www import WWW

TEST_DATA_LIST = [
    {'name': 'Alpha', 'age': '1'},
    {'name': 'Bravo', 'age': '2'},
]

TEST_DATA_ITEM_LIST = [
    True,
    1234,
    1234.5678,
    '1234',
    [1, 2, 3, 'test'],
    {'test': 123},
    TEST_DATA_LIST,
]


class TestFile(unittest.TestCase):
    def test_eq(self):
        file1 = File(
            tempfile.NamedTemporaryFile(prefix="test1.", suffix=".txt").name
        )
        file2 = File(
            tempfile.NamedTemporaryFile(prefix="test2.", suffix=".txt").name
        )
        self.assertEqual(file1, file1)
        self.assertNotEqual(file1, file2)
        self.assertNotEqual(file1, 'file2')

    def test_read_and_write(self):
        """Test."""
        content = 'Hello' * 100
        file = File(
            tempfile.NamedTemporaryFile(
                prefix="utils.test_file.", suffix=".txt"
            ).name
        )
        file.write(content)
        content2 = file.read()
        self.assertEqual(content, content2)
        self.assertEqual(file.ext, 'txt')

        lines = [f'Hello {i}' for i in range(0, 100)]
        file.write_lines(lines)
        lines2 = file.read_lines()
        self.assertEqual(lines, lines2)

    def test_json_read_and_write(self):
        for data in TEST_DATA_ITEM_LIST:
            json_file = JSONFile(
                tempfile.NamedTemporaryFile(
                    prefix="utils.test_file.", suffix=".json"
                ).name
            )
            json_file.write(data)
            data2 = json_file.read()
            self.assertEqual(data, data2)

    def test_xsv_delimiter(self):
        xsv_file = XSVFile('')
        with self.assertRaises(NotImplementedError) as _:
            xsv_file.delimiter

    def test_xsv_read_helper(self):
        delimiter = ' '
        xsv_lines = ['a b c', '1 2 3']
        data_list = XSVFile._readHelper(delimiter, xsv_lines)
        expected_data_list = [{'a': '1', 'b': '2', 'c': '3'}]
        self.assertEqual(expected_data_list, data_list)

    def test_csv_delimiter(self):
        csv_file = CSVFile('')
        self.assertEqual(csv_file.delimiter, ',')

    def test_csv_read_and_write(self):
        csv_file = CSVFile(
            tempfile.NamedTemporaryFile(
                prefix="utils_test_file.", suffix=".csv"
            ).name
        )
        csv_file.write(TEST_DATA_LIST)
        data_list = csv_file.read()
        self.assertEqual(TEST_DATA_LIST, data_list)

    def test_tsv_read_and_write(self):
        tsv_file = TSVFile(
            tempfile.NamedTemporaryFile(
                prefix="utils_test_file.", suffix=".tsv"
            ).name
        )
        tsv_file.write(TEST_DATA_LIST)
        data_list = tsv_file.read()
        self.assertEqual(TEST_DATA_LIST, data_list)

    def test_tsv_read_and_write_custom(self):
        URL_SUMMARY = urljoin(
            'https://raw.githubusercontent.com',
            'nuuuwan/news_lk3_data/main/reports/summary.tsv',
        )
        tmp_file = WWW(URL_SUMMARY).download()
        d_list = TSVFile(tmp_file).read()
        tsv_file = TSVFile(
            tempfile.NamedTemporaryFile(
                prefix="utils_test_file.", suffix=".tsv"
            ).name
        )
        tsv_file.write(d_list)
        data_list2 = tsv_file.read()
        self.assertEqual(d_list, data_list2)

    def write_test_json_file(self):
        json_file_name = tempfile.NamedTemporaryFile(
            prefix="utils.test_zip_read_and_write.", suffix=".json"
        ).name
        data = [i for i in range(0, 1_000)]
        json_file = JSONFile(json_file_name)
        json_file.write(data)
        json_file_size = os.path.getsize(json_file_name)
        expected_json_file_size = 6_892
        self.assertEqual(expected_json_file_size, json_file_size)
        return json_file_name, json_file, data


if __name__ == '__main__':
    unittest.main()



import os

from utils.file.FileOrDirectory import FileOrDirectory

DIALECT = 'excel'
DELIMITER_CSV = ','
DELIMITER_TSV = '\t'
DELIM_LINE = '\n'
ENCODING = 'utf-8'


class File(FileOrDirectory):
    def delete(self):
        if self.exists:
            os.remove(self.path)

    @property
    def ext(self):
        return self.name.split('.')[-1]

    def read(self):
        with open(self.path, 'r', encoding=ENCODING) as fin:
            content = fin.read()
            fin.close()
        return content

    def readBinary(self):
        with open(self.path, 'rb') as fin:
            content = fin.read()
            fin.close()
        return content

    def write(self, content):
        with open(self.path, 'w', encoding=ENCODING) as fout:
            fout.write(content)
            fout.close()

    def writeBinary(self, content):
        with open(self.path, 'wb') as fout:
            fout.write(content)
            fout.close()

    def read_lines(self):
        content = File.read(self)
        return content.split(DELIM_LINE)

    def write_lines(self, lines):
        content = DELIM_LINE.join(lines)
        File.write(self, content)



from utils.time.Time import Time
from utils.time.TimeDelta import TimeDelta
from utils.time.TimeFormat import (TIME_FORMAT_DATE, TIME_FORMAT_DATE_ID,
                                   TIME_FORMAT_TIME, TIME_FORMAT_TIME_ID,
                                   TimeFormat)
from utils.time.TimeID import get_date_id, get_time_id
from utils.time.TimeUnit import DAYS_IN, SECONDS_IN, TimeUnit
from utils.time.TIMEZONE_OFFSET import TIMEZONE_OFFSET



"""Tests for hashx"""

import unittest

from utils import hashx


class TestHashx(unittest.TestCase):
    """Tests."""

    def test_md5(self):
        """Test."""
        self.assertEqual(
            'fc3ff98e8c6a0d3087d515c0473f8677',
            hashx.md5('hello world!'),
        )


if __name__ == '__main__':
    unittest.main()



import os
import shutil

from utils.Log import Log

log = Log('Git')


class GitReadOnly:
    def __init__(self, git_repo_url):
        self.git_repo_url = git_repo_url
        self.repo_name = os.path.basename(git_repo_url)
        self.dir_repo = None
        self.branch_name = None

    def clone(self, dir_repo, force=False):
        self.dir_repo = dir_repo
        self.force = force

    def init_dir_repo(self):
        if os.path.exists(self.dir_repo):
            shutil.rmtree(self.dir_repo)
        os.mkdir(self.dir_repo)

    @property
    def cmd_clone(self):
        assert self.git_repo_url is not None
        assert self.dir_repo is not None
        assert self.branch_name is not None

        return ''.join(
            [
                f'git clone  -b {self.branch_name} --single-branch ',
                f' {self.git_repo_url} {self.dir_repo}',
            ]
        )

    @property
    def cmd_cd(self):
        assert self.dir_repo is not None
        return f'cd {self.dir_repo}'

    @property
    def cmd_git_pull(self):
        assert self.branch_name is not None
        return f'git pull origin {self.branch_name}'

    @property
    def cmd_checkout(self):
        assert self.branch_name is not None
        return f'git checkout {self.branch_name}'

    @staticmethod
    def run(*cmd_list):
        cmd = ' && '.join(cmd_list)
        log.debug(f'Running {cmd}')
        os.system(cmd)

    def checkout(self, branch_name):
        self.branch_name = branch_name

        if not os.path.exists(self.dir_repo) or self.force:
            self.init_dir_repo()
            self.run(
                self.cmd_cd,
                self.cmd_clone,
            )
        else:
            log.debug(f'{self.dir_repo} exists. Not cloning!')

        self.run(
            self.cmd_cd,
            self.cmd_checkout,
            self.cmd_git_pull,
        )

    def add_and_commit(self, message):
        self.run(self.cmd_cd, 'git add .', f'git commit -m "{message}"')



"""Test."""
import io
import tempfile
import unittest

from utils.xmlx import _, render_link_styles, style

TEST_BODY = _(
    'body',
    [
        _('h1', 'This is header 1', style(font_family='Georgia')),
        _('p', 'This is a paragraph'),
        _(
            'div',
            [
                _('span', 'This is a span in a div'),
                _(
                    'a',
                    'This is a a(link) in a div',
                    dict(href='https://pypi.org/project/utils-nuuuwan'),
                ),
            ],
        ),
    ],
)


class TestXMLX(unittest.TestCase):
    """Test."""

    def test_render_link_styles(self):
        """Test."""
        expected = '''<?xml version="1.0" ?>
<link rel="stylesheet" href="styles.css"/>
'''
        actual = str(render_link_styles())
        self.assertEqual(expected, actual)

    def test_repr(self):
        """Test."""
        expected = '<?xml version="1.0" ?>\n<p>This is a paragraph</p>\n'

        actual = repr(_('p', 'This is a paragraph'))
        self.assertEqual(expected, actual)

    def test_log_metric(self):
        """Test."""
        head = _('head')
        body = TEST_BODY
        html = _('html', [head, body])
        actual_file = tempfile.NamedTemporaryFile(
            prefix="utils.tests.test_xmlx.", suffix=".html"
        ).name
        html.store(actual_file)

        expected_file = 'tests/test_xmlx_example1.html'
        self.assertListEqual(
            list(io.open(actual_file)),
            list(io.open(expected_file)),
        )



"""Implements twitter."""


from utils.Log import Log
from utils.time.Time import Time
from utils.time.TimeFormat import TimeFormat
from utils.time.TIMEZONE_OFFSET import TIMEZONE_OFFSET

log = Log('Twitter')


def _update_status(api, tweet_text, media_ids, in_reply_to_status_id):
    if len(media_ids) > 0:
        if in_reply_to_status_id:
            response = api.update_status(
                tweet_text,
                media_ids=media_ids,
                in_reply_to_status_id=in_reply_to_status_id,
            )
        else:
            response = api.update_status(
                tweet_text,
                media_ids=media_ids,
            )
    else:
        if in_reply_to_status_id:
            response = api.update_status(
                tweet_text, in_reply_to_status_id=in_reply_to_status_id
            )
        else:
            response = api.update_status(tweet_text)
    return response


def _upload_media(api, image_files):
    media_ids = []
    for image_file in image_files:
        media_id = api.media_upload(image_file).media_id
        media_ids.append(media_id)
        log.info(
            f'Uploaded status image {image_file} to twitter as {media_id}',
        )
    return media_ids


def _update_profile_description(api):
    date_with_timezone = TimeFormat(
        '%Y-%m-%d %H:%M:%S', TIMEZONE_OFFSET.LK
    ).stringify(Time.now())
    description = (
        f'Automatically updated at {date_with_timezone} (#SriLanka Time)'
    )
    api.update_profile(description=description)
    log.info(f'Updated profile description to: {description}')



import csv

from utils.file.File import File

DIALECT = 'excel'


class XSVFile(File):
    @property
    def delimiter(self):
        raise NotImplementedError

    @staticmethod
    def _readHelper(delimiter: str, xsv_lines: list):
        data_list = []
        field_names = None
        reader = csv.reader(
            xsv_lines,
            dialect=DIALECT,
            delimiter=delimiter,
        )
        for row in reader:
            if not field_names:
                field_names = row
            else:
                data = dict(
                    zip(
                        field_names,
                        row,
                    )
                )
                if data:
                    data_list.append(data)
        return data_list

    def read(self):
        xsv_lines = File.read_lines(self)
        return XSVFile._readHelper(self.delimiter, xsv_lines)

    @staticmethod
    def get_field_names(data_list):
        return list(data_list[0].keys())

    @staticmethod
    def get_data_rows(data_list, field_names):
        return list(
            map(
                lambda data: list(
                    map(
                        lambda field_name: data[field_name],
                        field_names,
                    )
                ),
                data_list,
            )
        )

    def write(self, data_list):
        with open(self.path, 'w', encoding="utf8") as fout:
            writer = csv.writer(
                fout,
                dialect=DIALECT,
                delimiter=self.delimiter,
            )

            field_names = XSVFile.get_field_names(data_list)
            writer.writerow(field_names)
            writer.writerows(XSVFile.get_data_rows(data_list, field_names))
            fout.close()



import os
from unittest import TestCase

from utils import Directory, File


class TestDirectory(TestCase):
    def test_init(self):
        dir_tests = Directory(os.path.join('src', 'utils'))
        self.assertEqual(dir_tests.path, os.path.join('src', 'utils'))
        self.assertEqual(dir_tests.name, 'utils')
        self.assertEqual(
            dir_tests.children[0],
            File(os.path.join('src', 'utils', 'Browser.py')),
        )

        self.assertEqual(dir_tests, Directory(os.path.join('src', 'utils')))



"""Test."""
import unittest

from utils.String import String

SNAKE_AND_CAMEL_EXAMPLES = [
    ['this_is_a_test', 'ThisIsATest'],
    ['123', '123'],
    ['123_testing123', '123Testing123'],
]


class TestDT(unittest.TestCase):
    def test_float(self):
        for [input, expected_output] in [
            ['0', 0],
            ['123', 123],
            ['123abc', None],
            ['123.456', 123.456],
        ]:
            self.assertEqual(
                String(input).float,
                expected_output,
            )

    def test_int(self):
        for [input, expected_output] in [
            ['0', 0],
            ['123', 123],
            ['123abc', None],
            ['123.456', 123],
        ]:
            self.assertEqual(
                String(input).int,
                expected_output,
            )

    def test_snake(self):
        for [input, expected_output] in [
            ['This is a test', 'this_is_a_test'],
            ['123', '123'],
            ['123 Testing 123', '123_testing_123'],
        ]:
            self.assertEqual(
                String(input).snake,
                expected_output,
            )

    def test_kebab(self):
        for [input, expected_output] in [
            ['This is a test', 'this-is-a-test'],
            ['123', '123'],
            ['123 Testing 123', '123-testing-123'],
        ]:
            self.assertEqual(
                String(input).kebab,
                expected_output,
            )

    def test_camel(self):
        for [input, expected_output] in [
            ['This is a test', 'ThisIsATest'],
            ['123', '123'],
            ['123 Testing 123', '123Testing123'],
        ]:
            self.assertEqual(
                String(input).camel,
                expected_output,
            )

    def test_str_and_repr(self):
        for input in [
            'This is a test',
            '123',
            '123 Testing 123',
        ]:
            self.assertEqual(
                str(String(input)),
                input,
            )
            self.assertEqual(
                repr(String(input)),
                input,
            )



import time

from utils.time.Time import Time
from utils.time.TIMEZONE_OFFSET import TIMEZONE_OFFSET


class TimeFormat:
    def __init__(self, format_str: str, timezone_offset=TIMEZONE_OFFSET.LK):
        self.format_str = format_str
        self.timezone_offset = timezone_offset

    @property
    def dut_timezone(self):
        return time.timezone - self.timezone_offset

    def parse(self, time_str: str) -> Time:
        ut_base = time.mktime(time.strptime(time_str, self.format_str))
        ut = ut_base - self.dut_timezone
        return Time(ut)

    def stringify(self, t: Time) -> str:
        return time.strftime(
            self.format_str, time.localtime(t.ut + self.dut_timezone)
        )


TIME_FORMAT_DATE = TimeFormat('%Y-%m-%d')
TIME_FORMAT_TIME = TimeFormat('%Y-%m-%d %H:%M:%S')

TIME_FORMAT_DATE_ID = TimeFormat('%Y%m%d')
TIME_FORMAT_TIME_ID = TimeFormat('%Y%m%d.%H%M%S')



from unittest import TestCase

from utils.Table import Table, TableRow

TEST_DICT_LIST = [
    dict(name='Alice', age=30),
    dict(name='Bob', age=40),
    dict(name='Charlie', age=23),
    dict(name='Dinah', age=19),
]
TEST_TABLE = Table(TEST_DICT_LIST)
TEST_ROW = TableRow(TEST_DICT_LIST[0])


class TestTable(TestCase):
    # TableRow
    def test_row_to_dict(self):
        self.assertEqual(TEST_ROW.to_dict(), TEST_DICT_LIST[0])

    def test_row_eq(self):
        self.assertEqual(TEST_ROW, TableRow(TEST_DICT_LIST[0]))

        self.assertNotEqual(TEST_ROW, "TableRow")

    def test_row_fields(self):
        self.assertEqual(TEST_ROW.fields, ['name', 'age'])

    def test_row_getattr(self):
        self.assertEqual(TEST_ROW.name, TEST_DICT_LIST[0]['name'])

    def test_from_dict(self):
        self.assertEqual(TableRow.from_dict(TEST_DICT_LIST[0]), TEST_ROW)

    # Table
    def test_init(self):
        self.assertEqual(Table(TEST_DICT_LIST).to_dict_list(), TEST_DICT_LIST)
        self.assertEqual(TEST_TABLE.to_dict_list(), TEST_DICT_LIST)
        self.assertEqual(
            Table.from_dict(TEST_DICT_LIST).to_dict_list(), TEST_DICT_LIST
        )

    def test_len(self):
        self.assertEqual(len(TEST_TABLE), 4)

    def test_filter(self):
        self.assertEqual(
            TEST_TABLE.filter(lambda d: d['age'] > 30),
            Table([dict(name='Bob', age=40)]),
        )

    def test_fields(self):
        self.assertEqual(TEST_TABLE.fields, ['name', 'age'])

    def test_getitem(self):
        self.assertEqual(TEST_TABLE[0], TableRow(TEST_DICT_LIST[0]))

    def test_add(self):
        self.assertEqual(
            Table(TEST_DICT_LIST[:2]) + Table(TEST_DICT_LIST[2:4]),
            TEST_TABLE,
        )

    def test_map(self):
        self.assertEqual(
            TEST_TABLE.map(lambda d: dict(name=d['name'])),
            Table(
                [
                    {'name': 'Alice'},
                    {'name': 'Bob'},
                    {'name': 'Charlie'},
                    {'name': 'Dinah'},
                ]
            ),
        )

    def test_eq(self):
        self.assertEqual(TEST_TABLE, Table(TEST_DICT_LIST))
        self.assertNotEqual(TEST_TABLE, "Table")



from unittest import TestCase

from utils.List import List


class TestIter(TestCase):
    def test_count_list1(self):
        x = List(
            [
                'Nuwan Senaratna',
                'Nuwan Silva',
                'Nuwan de Silva',
                'Albert Einstein',
                'Albert Perera',
                'Barbie Perera',
            ]
        )

        def func_key(x):
            return x.split(' ')[0]

        self.assertEqual(
            x.count(func_key),
            {'Nuwan': 3, 'Albert': 2, 'Barbie': 1},
        )

    def test_count_list2(self):
        x = List([1, 1, 2, 2, 2, 3, 3, 3, 3, 4, 4, 4])
        self.assertEqual(x.count(), {1: 2, 2: 3, 3: 4, 4: 3})

    def test_count_list3(self):
        int_list = [(i + 1) for i in range(0, 10000)]
        x = List(int_list)

        def func_key(i):
            return i % 3

        self.assertEqual(x.count(func_key), {0: 3333, 1: 3334, 2: 3333})



from utils.Iter import Iter


class List(Iter):
    def __init__(self, x=[]):
        assert isinstance(x, list)
        self.x = x

    def tolist(self):
        return self.x

    def __iter__(self):
        return self.x.__iter__()

    def __len__(self):
        return len(self.x)

    def __getitem__(self, idx):
        return self.x[idx]

    def __setitem__(self, idx, value):
        self.x[idx] = value

    def __eq__(self, other):
        if isinstance(other, List):
            return self.x == other.x
        if isinstance(other, list):
            return self.x == other
        return False

    def __str__(self):
        return str(self.x)

    def __repr__(self):
        return repr(self.x)

    def __add__(self, other):
        assert isinstance(other, List)
        return List(self.x + other.x)

    def flatten(self):
        lst2 = []
        for lst_inner in self.x:
            if not isinstance(lst_inner, list):
                raise TypeError("List.flatten: list contains non-lists")
            lst2 += lst_inner
        return List(lst2)

    def unique(self):
        return List(list(set(self.x)))



from utils.Log import Log

log = Log('Git')


class GitWrite:
    @property
    def cmd_git_push(self):
        assert self.branch_name is not None
        return f'git push origin {self.branch_name}'

    def push(self):
        self.run(
            self.cmd_cd,
            self.cmd_git_push,
        )



from utils.twitter.TwitterActionerMixin import TwitterActionerMixin
from utils.twitter.TwitterBase import TwitterBase
from utils.twitter.TwitterLoaderMixin import TwitterLoaderMixin


class Twitter(TwitterBase, TwitterLoaderMixin, TwitterActionerMixin):
    pass



"""Utils for reading remote files."""
import os
import ssl
import tempfile

import requests
from bs4 import BeautifulSoup
from selenium import webdriver
from selenium.webdriver.firefox.options import Options

from utils import hashx
from utils.file.CSVFile import CSVFile
from utils.file.File import File
from utils.file.JSONFile import JSONFile
from utils.file.TSVFile import TSVFile
from utils.Log import Log

log = Log('WWW')

USER_AGENT = ''.join(
    [
        'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.14; rv:65.0) ',
        'Gecko/20100101 Firefox/65.0',
    ]
)

ENCODING = 'utf-8'
SELENIUM_SCROLL_REPEATS = 3
SELENIUM_SCROLL_WAIT_TIME = 0.5
EXISTS_TIMEOUT = 1

HASH_LENGTH = 8

BINARY_EXT_LIST = ['pdf', 'png', 'jpg', 'jpeg']
NON_BINARY_EXT_LIST = ['json', 'tsv', 'csv', 'txt']
HTML_EXT_LIST = ['htm', 'html']
CUSTOM_EXT_LIST = BINARY_EXT_LIST + NON_BINARY_EXT_LIST

# pylint: disable=W0212
ssl._create_default_https_context = ssl._create_unverified_context


class WWW:
    def __init__(self, url: str):
        self.url = url

    @property
    def hash_id(self):
        return hashx.md5(self.url)[:HASH_LENGTH]

    @property
    def ext(self):
        for ext in CUSTOM_EXT_LIST + HTML_EXT_LIST:
            if self.url.endswith(ext):
                return ext
        return 'htm'

    @property
    def local_path(self):
        return os.path.join(
            tempfile.gettempdir(), f'www.{self.hash_id}.{self.ext}'
        )

    def read_html(self):
        options = Options()
        options.add_argument('-headless')
        driver = webdriver.Firefox(options=options)

        try:
            driver.get(self.url)
        except BaseException:
            log.error(f'Failed to read html: {self.url}')
            return None

        content = driver.page_source
        driver.quit()
        return content

    def download_html(self):
        html = self.read_html()
        File(self.local_path).write(html)
        return self.local_path

    @staticmethod
    def download_binary(url, file_path):
        CHUNK_SIZE = 1024
        r = requests.get(url, stream=True)
        with open(file_path, 'wb') as fd:
            for chunk in r.iter_content(CHUNK_SIZE):
                fd.write(chunk)
        return file_path

    def download(self):
        if os.path.exists(self.local_path):
            log.debug(f'local path exists: {self.local_path}')
            return self.local_path

        if not self.exists:
            raise Exception(f'WWW does not exist: {self.url}')

        if self.ext in HTML_EXT_LIST:
            return self.download_html()
        return WWW.download_binary(self.url, self.local_path)

    def read(self):
        self.download()
        if self.ext in NON_BINARY_EXT_LIST + HTML_EXT_LIST:
            return File(self.local_path).read()
        return File(self.local_path).readBinary()

    @property
    def exists(self):
        try:
            response = requests.head(self.url, timeout=EXISTS_TIMEOUT)
            # pylint: disable=E1101
            return response.status_code == requests.codes.ok
        except BaseException:
            return False

    @property
    def children(self):
        soup = self.soup
        links = soup.find_all('a')
        raw_urls = [WWW(link.get('href')) for link in links]
        raw_urls = list(filter(lambda x: x.url, raw_urls))
        return list(sorted(set(raw_urls), key=lambda x: x.url))

    @property
    def soup(self):
        if self.ext not in HTML_EXT_LIST:
            log.warning(f'Cannot extract soup from non-html file: {self.url}')
        return BeautifulSoup(self.read(), 'html.parser')

    # -----------
    # Legacy methods (should be deprecated)
    # -----------
    def readJSON(self):
        return JSONFile(self.download()).read()

    def readTSV(self):
        return TSVFile(self.download()).read()

    def readCSV(self):
        return CSVFile(self.download()).read()

    def readBinary(self):
        return File(self.download()).readBinary()

    def readSelenium(self):
        return self.read_html()



import tweepy

from utils.Log import Log

log = Log('Twitter')


class TwitterBase:
    twtr_api_key: str
    twtr_api_secret_key: str
    twtr_access_token: str
    twtr_access_token_secret: str

    def __init__(
        self,
        twtr_api_key,
        twtr_api_secret_key,
        twtr_access_token,
        twtr_access_token_secret,
    ):
        self.twtr_api_key = twtr_api_key
        self.twtr_api_secret_key = twtr_api_secret_key
        self.twtr_access_token = twtr_access_token
        self.twtr_access_token_secret = twtr_access_token_secret

        self.api = self.init_api()

    def init_api(self):
        try:
            auth = tweepy.OAuthHandler(
                self.twtr_api_key, self.twtr_api_secret_key
            )
            log.debug('Created Twitter API auth.')
            auth.set_access_token(
                self.twtr_access_token, self.twtr_access_token_secret
            )
            log.debug('Set Twitter Access Token.')
            return tweepy.API(auth)
        except Exception as e:
            log.error(f'Twitter API Setup failed: {e}')
            return None



import time
from unittest import TestCase

from utils import FiledVariable

MAX_N = 1_000_000
VALUE = (MAX_N) * (MAX_N - 1) // 2


def func_get():
    s = 0
    for i in range(MAX_N):
        s += i
    return s


class TestFiledVariable(TestCase):
    def test_cache_key(self):
        fv = FiledVariable('test_key', None)
        self.assertEqual(fv.cache_key, '8c32d1183251df9828f929b935ae0419')

    def test_cache_get(self):
        fvar = FiledVariable('test_key', func_get)
        fvar.clear_file()
        fvar.clear_cache()
        self.assertEqual(fvar.key, 'test_key')
        self.assertEqual(fvar.value, VALUE)

        t0 = time.time()
        self.assertEqual(fvar.value, VALUE)
        dt = time.time() - t0
        self.assertLess(dt, 0.001)

    def test_file_get(self):
        fvar = FiledVariable('test_key', func_get)
        fvar.clear_cache()
        self.assertEqual(fvar.value, VALUE)



from unittest import TestCase

from utils.Dict import Dict

TEST_DICT_RAW = dict(a=1, b=2, c=3, d=4)
TEST_DICT = Dict(TEST_DICT_RAW)


class TestDict(TestCase):
    def test_init(self):
        self.assertEqual(TEST_DICT.todict(), TEST_DICT_RAW)
        x = Dict()
        self.assertEqual(x.todict(), {})

    def test_keys(self):
        self.assertEqual(TEST_DICT.keys(), TEST_DICT_RAW.keys())

    def test_values(self):
        self.assertEqual(
            list(TEST_DICT.values()), list(TEST_DICT_RAW.values())
        )

    def test_items(self):
        self.assertEqual(TEST_DICT.items(), TEST_DICT_RAW.items())

    def test_len(self):
        self.assertEqual(TEST_DICT.len(), len(TEST_DICT_RAW))

    def test_eq(self):
        self.assertTrue(TEST_DICT == TEST_DICT_RAW)
        self.assertTrue(TEST_DICT == Dict(TEST_DICT_RAW))
        self.assertFalse(
            Dict({'a': 3, 'b': 2, 'c': 1}) == {'a': 3, 'b': 2, 'c': 14}
        )
        self.assertFalse(
            Dict({'a': 3, 'b': 2, 'c': 1}) == Dict({'a': 3, 'b': 2, 'c': 14})
        )
        self.assertFalse(TEST_DICT == 1)

    def test_getitem(self):
        for k in TEST_DICT_RAW.keys():
            self.assertEqual(TEST_DICT[k], TEST_DICT_RAW[k])

    def test_setitem(self):
        x = Dict()
        x['a'] = 11
        x['b'] = 22
        self.assertEqual(x.todict(), {'a': 11, 'b': 22})

    def test_extract_keys(self):
        self.assertEqual(TEST_DICT.extract_keys(['a']), Dict({'a': 1}))

    def test_items_sorted_by_key(self):
        self.assertEqual(
            Dict({'c': 3, 'b': 2, 'a': 1}).items_sorted_by_key(),
            [('a', 1), ('b', 2), ('c', 3)],
        )

    def test_items_sorted_by_value(self):
        self.assertEqual(
            Dict({'a': 3, 'b': 2, 'c': 1}).items_sorted_by_value(),
            [('c', 1), ('b', 2), ('a', 3)],
        )

    def test_del(self):
        d = Dict({'a': 3, 'b': 2, 'c': 1})
        del d['a']
        self.assertEqual(d, Dict({'b': 2, 'c': 1}))

    def test_iter(self):
        k_list = []
        for k in TEST_DICT:
            k_list.append(k)
        self.assertEqual(k_list, list(TEST_DICT_RAW.keys()))

    def test_str(self):
        self.assertEqual(str(TEST_DICT), str(TEST_DICT_RAW))

    def test_repr(self):
        self.assertEqual(Dict.__repr__(TEST_DICT), str(TEST_DICT_RAW))



import tempfile
from unittest import TestCase

from selenium.webdriver.common.by import By

from utils import Browser

TEST_URL = '/'.join(
    [
        'https://nuuuwan.github.io',
        'utils',
    ]
)


class TestBrowser(TestCase):
    def test_find_element_etc(self):
        browser = Browser()
        browser.open(TEST_URL)
        elem_h1 = browser.find_element(By.TAG_NAME, 'h1')
        self.assertEqual('Heading 1', elem_h1.text)

        elem_h2_2 = browser.find_elements(By.TAG_NAME, 'h2')[1]
        self.assertEqual('Heading 1.2', elem_h2_2.text)

        elem_h2_1 = browser.wait_for_element(By.TAG_NAME, 'h2')
        self.assertEqual('Heading 1.1', elem_h2_1.text)

        browser.quit()

    def test_source(self):
        browser = Browser()
        browser.open(TEST_URL)
        self.assertIn('This is a test', browser.source)
        browser.quit()

    def test_all_others(self):
        browser = Browser()
        browser.open(TEST_URL)
        browser.set_window_dim((100, 200))
        browser.scroll_to_bottom()
        browser.downloadScreenshot(
            tempfile.NamedTemporaryFile(
                prefix="screenshot.", suffix=".png"
            ).name
        )
        browser.sleep(1)
        browser.quit()



import time

from selenium import webdriver
from selenium.webdriver.firefox.options import Options
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.wait import WebDriverWait

from utils.Log import Log

DEFAULT_WINDOW_DIM = (1920, 1080)
TIME_DEFAULT_WAIT = 10
TIME_DEFAULT_SLEEP = 1

log = Log('Browser')


class Browser:
    """Browser."""

    def __init__(self):
        """Construct."""
        options = Options()
        options.headless = True
        self.driver = webdriver.Firefox(
            options=options,
        )
        self.set_window_dim(DEFAULT_WINDOW_DIM)

    def set_window_dim(self, dim: tuple):
        width, height = dim
        self.driver.set_window_size(width, height)

    def open(self, url: str):
        self.driver.get(url)
        log.debug(f'Opened "{url}".')

    def scroll_to_bottom(self):
        """Scroll to the bottom of the page."""
        SCRIPT_SCROLL = 'window.scrollTo(0, document.body.scrollHeight);'
        self.driver.execute_script(SCRIPT_SCROLL)

    @property
    def source(self):
        """Get page source."""
        return self.driver.page_source

    def downloadScreenshot(self, image_file_name):
        self.driver.save_screenshot(image_file_name)
        log.debug(f'Downloaded screenshot to "{image_file_name}".')

    def find_element(self, by, value):
        return self.driver.find_element(by, value)

    def find_elements(self, by, value):
        return self.driver.find_elements(by, value)

    def wait_for_element(self, by, value, timeout: int = TIME_DEFAULT_WAIT):
        return WebDriverWait(self.driver, timeout).until(
            EC.presence_of_element_located((by, value))
        )

    def sleep(self, timeout=TIME_DEFAULT_SLEEP):
        time.sleep(timeout)

    def quit(self):
        self.driver.close()
        self.driver.quit()



import os
import unittest

from utils import WWW, CSVFile, File, JSONFile, TSVFile

DIR_TESTS = 'tests'


URL_BASE = '/'.join(
    [
        'https://raw.githubusercontent.com',
        'nuuuwan/utils',
        'main/tests',
    ]
)

URL_HTML = 'https://nuuuwan.github.io/utils/index.html'


def get_test_file(ext: str) -> str:
    return os.path.join(DIR_TESTS, f'data.{ext}')


def strip_html(html):
    return html.replace('\t', '').replace('\n', '').replace(' ', '')


def get_test_url(ext: str) -> str:
    if ext == 'html':
        return URL_HTML
    return f'{URL_BASE}/data.{ext}'


def cleanup_local_files():
    pass


class TestWWW(unittest.TestCase):
    def test_download_html(self):
        cleanup_local_files()
        for _ in range(2):
            self.assertEqual(
                strip_html(File(get_test_file('html')).read()),
                strip_html(WWW(get_test_url('html')).read()),
            )

    def test_download_binary(self):
        cleanup_local_files()
        self.assertEqual(
            File(get_test_file('png')).readBinary(),
            WWW(get_test_url('png')).read(),
        )

    def test_read(self):
        cleanup_local_files()
        self.assertEqual(
            File(get_test_file('txt')).read(),
            WWW(get_test_url('txt')).read(),
        )

    def test_exists(self):
        cleanup_local_files()
        url = get_test_url('png')
        self.assertTrue(WWW(url).exists)
        self.assertFalse(WWW(url + '.1234').exists)

    def test_children(self):
        cleanup_local_files()
        url = 'https://www.python.org/'
        children = WWW(url).children
        self.assertGreater(len(children), 0)
        print(children[0].url)
        self.assertIn(children[0].url, '#')

    # ----------------------------
    # To Deprecate
    # ----------------------------

    def test_read_json(self):
        cleanup_local_files()
        self.assertEqual(
            JSONFile(get_test_file('json')).read(),
            WWW(get_test_url('json')).readJSON(),
        )

    def test_read_tsv(self):
        cleanup_local_files()
        self.assertEqual(
            TSVFile(get_test_file('tsv')).read(),
            WWW(get_test_url('tsv')).readTSV(),
        )

    def test_read_csv(self):
        cleanup_local_files()
        self.assertEqual(
            CSVFile(get_test_file('csv')).read(),
            WWW(get_test_url('csv')).readCSV(),
        )

    def test_read_binary(self):
        cleanup_local_files()
        self.assertEqual(
            File(get_test_file('png')).readBinary(),
            WWW(get_test_url('png')).readBinary(),
        )

        with self.assertRaises(Exception):
            WWW(get_test_url('png') + '.1234').readBinary()

    def test_read_selenium(self):
        cleanup_local_files()
        content = WWW(get_test_url('html')).readSelenium()
        self.assertIn(
            'This is a test',
            content,
        )



import os
from unittest import TestCase

from utils import PDFFile

TEST_PDF_FILE = PDFFile(os.path.join('tests', 'example.pdf'))


class TestPDFFile(TestCase):
    def test_n_pages(self):
        self.assertEqual(TEST_PDF_FILE.n_pages, 16)



import os


class FileOrDirectory:
    def __init__(self, path):
        self.path = path

    @property
    def name(self):
        return self.path.split(os.sep)[-1]

    @property
    def exists(self):
        return os.path.exists(self.path)

    def __eq__(self, other):
        if isinstance(self, type(other)):
            return self.path == other.path
        return False



from unittest import TestCase

from utils.List import List

TEST_LIST_RAW = ['a', 'b', 'c', 'd']
TEST_LIST = List(TEST_LIST_RAW)


class TestList(TestCase):
    def test_init(self):
        self.assertEqual(TEST_LIST.tolist(), TEST_LIST_RAW)
        x = List()
        self.assertEqual(x.tolist(), [])

    def test_len(self):
        self.assertEqual(len(TEST_LIST), len(TEST_LIST_RAW))

    def test_getitem(self):
        for idx in range(len(TEST_LIST_RAW)):
            self.assertEqual(TEST_LIST[idx], TEST_LIST_RAW[idx])

    def test_setitem(self):
        x = List([1, 2])
        x[0] = 11
        x[1] = 22
        self.assertEqual(x.tolist(), [11, 22])

    def test_eq(self):
        self.assertEqual(TEST_LIST, TEST_LIST_RAW)
        self.assertEqual(TEST_LIST, List(TEST_LIST_RAW))
        self.assertNotEqual(TEST_LIST, "List")

    def test_str(self):
        self.assertEqual(str(TEST_LIST), str(TEST_LIST_RAW))

    def test_repr(self):
        self.assertEqual(repr(TEST_LIST), repr(TEST_LIST_RAW))

    def test_add(self):
        self.assertEqual(List([1, 2]) + List([3, 4]), List([1, 2, 3, 4]))

    def test_flatten(self):
        self.assertEqual(List([[1, 2], [3, 4]]).flatten(), List([1, 2, 3, 4]))
        with self.assertRaises(TypeError):
            List([[1, 2], 3]).flatten()

    def test_unique(self):
        self.assertEqual(List([1, 2, 3, 2]).unique(), List([1, 2, 3]))

    def test_iter(self):
        for item in TEST_LIST:
            self.assertIn(item, TEST_LIST_RAW)



"""Implements twitter."""


from utils.Log import Log
from utils.twitter.Tweet import Tweet
from utils.twitter.TwitterActionerMixinHelpers import (
    _update_profile_description, _update_status, _upload_media)

log = Log('Twitter')


class TwitterActionerMixin:
    def send(self, tweet: Tweet):
        media_ids = _upload_media(
            self.api,
            tweet.image_file_path_list,
        )
        return _update_status(
            self.api, tweet.text, media_ids, tweet.in_reply_to_status_id
        )

    def update_profile_description(self):
        _update_profile_description(self.api)

    def update_profile_image(self, profile_image_file):
        log.debug(f'update_profile_image: {profile_image_file}')
        self.api.update_profile_image(profile_image_file)

    def update_banner_image(self, banner_image_file):
        log.debug(f'update_banner_image: {banner_image_file}')
        self.api.update_profile_banner(banner_image_file)



import time
from unittest import TestCase

from utils import (TIMEZONE_OFFSET, Time, TimeDelta, TimeFormat, TimeUnit,
                   get_date_id, get_time_id)


class TestTime(TestCase):
    def test_init(self):
        ut = 1234567890
        t = Time(ut)
        self.assertEqual(t.ut, ut)

    def test_now(self):
        t = Time.now()
        ut = time.time()
        self.assertGreater(t.ut, ut - 10)
        self.assertGreater(ut + 10, t.ut)

        with self.assertRaises(TypeError):
            t + 1

    def test_sub(self):
        t0 = Time(1234567800)
        t1 = Time(1234567890)
        dut10 = TimeDelta(90)

        self.assertEqual(t1 - t0, dut10)
        self.assertEqual(t1 - dut10, t0)

        with self.assertRaises(TypeError):
            t0 - 1

    def test_add(self):
        t0 = Time(1234567800)
        dt = TimeDelta(100)
        t1 = Time(1234567900)
        self.assertEqual(t0 + dt, t1)

    def test_delta_humanize(self):
        for dut, expected_humanized in [
            [0, '0 seconds'],
            [1, '1 second'],
            [60, '1 minute'],
            [120, '2 minutes'],
            [3_600, '1 hour'],
            [86_400, '1 day'],
        ]:
            self.assertEqual(TimeDelta(dut).humanize(), expected_humanized)

    def test_format_stringify(self):
        t = Time(1234567890)
        for format_str, expected_time_str, ut2 in [
            ['%Y-%m-%d', '2009-02-14', 1234549800],
            ['%Y-%m-%d %H:%M', '2009-02-14 05:01', 1234567860],
            ['%Y-%m-%d %H:%M:%S', '2009-02-14 05:01:30', 1234567890],
        ]:
            tf = TimeFormat(format_str, TIMEZONE_OFFSET.LK)
            self.assertEqual(
                expected_time_str,
                tf.stringify(t),
            )

            t2 = Time(ut2)
            self.assertEqual(
                t2,
                tf.parse(expected_time_str),
            )

    def test_time_id(self):
        time_id = get_time_id()
        self.assertEqual(len(time_id), 15)

    def test_date_id(self):
        date_id = get_date_id()
        self.assertEqual(len(date_id), 8)

    def test_time_unit_truediv(self):
        self.assertEqual(TimeUnit(1) / 2, TimeUnit(0.5))
        self.assertEqual(TimeUnit(1) / TimeUnit(0.5), 2.0)

        with self.assertRaises(TypeError):
            2 / TimeUnit(2)

    def test_time_unit_mul(self):
        self.assertEqual(TimeUnit(1) * 2, TimeUnit(2))

    def test_time_unit_eq(self):
        self.assertEqual(TimeUnit(1) * 2, TimeUnit(2))
        self.assertNotEqual(TimeUnit(1) * 2, TimeUnit(3))
        self.assertNotEqual(TimeUnit(1) * 2, 2)



import random
from unittest import TestCase

from utils import Color

random.seed(0)


class TestColor(TestCase):
    def test_static_helpers(self):
        self.assertEqual(Color._8bit(), 48)
        self.assertEqual(Color._4bit_hex(), '8')
        self.assertEqual(Color._8bit_hex(), '49')
        self.assertEqual(Color._360(), 50)
        self.assertAlmostEqual(Color._float(), 0.7298317482601286)
        self.assertEqual(Color._percent(), 87)

    def test_rgba(self):
        self.assertEqual(Color.rgba(), 'rgba(111,71,144,0.14)')
        self.assertEqual(Color.rgba(12, 34, 56, 0.78), 'rgba(12,34,56,0.78)')

    def test_hsla(self):
        self.assertEqual(Color.hsla(), 'hsla(155,61%,45%,0.58)')
        self.assertEqual(
            Color.hsla(123, 45, 67, 0.89), 'hsla(123,45%,67%,0.89)'
        )

    def test_hex(self):
        self.assertEqual(Color.hex(), '#cd18fc')



import random


class Color:
    @staticmethod
    def _8bit():
        return random.randint(0, 255)

    @staticmethod
    def _4bit_hex():
        return random.choice('0123456789abcdef')

    @staticmethod
    def _8bit_hex():
        return Color._4bit_hex() + Color._4bit_hex()

    @staticmethod
    def _360():
        return random.randint(0, 355)

    @staticmethod
    def _float():
        return random.random()

    @staticmethod
    def _percent():
        return random.randint(0, 100)

    @staticmethod
    def rgba(r=None, g=None, b=None, a=None):
        r = r or Color._8bit()
        g = g or Color._8bit()
        b = b or Color._8bit()
        a = a or Color._float()
        return f'rgba({r:d},{g:d},{b:d},{a:.2f})'

    @staticmethod
    def hsla(h=None, s=None, light=None, a=None):
        h = h or Color._360()
        s = s or Color._percent()
        light = light or Color._percent()
        a = a or Color._float()
        return f'hsla({h:d},{s:d}%,{light:d}%,{a:.2f})'

    @staticmethod
    def hex():
        return '#' + ''.join([Color._8bit_hex() for _ in range(3)])



from utils.time.Time import Time
from utils.time.TimeFormat import TIME_FORMAT_DATE_ID, TIME_FORMAT_TIME_ID


def get_time_id():
    return TIME_FORMAT_TIME_ID.stringify(Time.now())


def get_date_id():
    return TIME_FORMAT_DATE_ID.stringify(Time.now())



"""Tests for utils."""

import unittest

from utils import Log, Twitter
from utils.twitter.TwitterActionerMixinHelpers import (
    _update_profile_description, _update_status, _upload_media)

log = Log('test_twitter')

TEST_QUOTE = '\n'.join(
    [
        'Data is a precious thing',
        'and will last longer',
        'than the systems themselves.',
        '- Tim Berners-Lee',
    ]
)


SKIP_API_CREDENTIALS = 'Needs Twitter API credentials'


class TestActionerMixinHelpers(unittest.TestCase):
    @unittest.skip(SKIP_API_CREDENTIALS)
    def test_update_status(self):
        twitter = Twitter.from_environ_vars()
        _update_status(twitter.api, TEST_QUOTE, [])

    @unittest.skip(SKIP_API_CREDENTIALS)
    def test_upload_media(self):
        twitter = Twitter.from_environ_vars()
        _upload_media(twitter.api, ['tests/data.png'])

    @unittest.skip(SKIP_API_CREDENTIALS)
    def test_update_profile_description(self):
        Twitter.from_environ_vars()
        _update_profile_description()



import re


class String:
    def __init__(self, s: str):
        self.s = s

    def __str__(self):
        return self.s

    def __repr__(self):
        return self.s

    @property
    def float(self, default=None):
        float_str = self.s
        float_str = float_str.replace(',', '')
        float_str = float_str.replace('-', '0')
        try:
            return (float)(float_str)
        except ValueError:
            return default

    @property
    def int(self, default=None):
        int_str = self.s
        int_str = int_str.replace(',', '')
        int_str = int_str.replace('-', '0')
        try:
            return (int)((float)(int_str))
        except ValueError:
            return default

    @property
    def kebab(self):
        s = self.s
        s = re.sub(r'[^a-zA-Z0-9]+', ' ', s)
        s = re.sub(r'\s+', ' ', s)
        s = s.replace(' ', '-')
        return s.lower()

    @property
    def snake(self):
        s = self.s
        return re.sub(r'(\s|-)+', '_', s).lower()

    @property
    def camel(self):
        s = self.snake
        return s.replace('_', ' ').title().replace(' ', '')



class TimeUnit:
    def __init__(self, seconds: int):
        self.seconds = seconds

    def __truediv__(self, other):
        if isinstance(other, TimeUnit):
            return self.seconds / other.seconds
        if isinstance(other, float) or isinstance(other, int):
            return TimeUnit(self.seconds / other)
        raise TypeError(
            f'unsupported operand type(s) for /: {type(self)} and {type(other)}'
        )

    def __mul__(self, other):
        return TimeUnit(self.seconds * other)

    def __eq__(self, other):
        if isinstance(other, TimeUnit):
            return self.seconds == other.seconds
        return False


SECOND = TimeUnit(1)
MINUTE = SECOND * 60
HOUR = MINUTE * 60
DAY = HOUR * 24
WEEK = DAY * 7
FORTNIGHT = WEEK * 7

AVG_YEAR = DAY * 365.25
AVG_QTR = AVG_YEAR / 4
AVG_MONTH = AVG_YEAR / 12


class SECONDS_IN:
    MINUTE = MINUTE / SECOND
    HOUR = HOUR / SECOND
    DAY = DAY / SECOND
    WEEK = WEEK / SECOND
    FORTNIGHT = FORTNIGHT / SECOND

    AVG_MONTH = AVG_MONTH / SECOND
    AVG_QTR = AVG_QTR / SECOND
    AVG_YEAR = AVG_YEAR / SECOND


class DAYS_IN:
    AVG_MONTH = AVG_MONTH / DAY
    AVG_QTR = AVG_QTR / DAY
    AVG_YEAR = AVG_YEAR / DAY



class TableRow:
    def __init__(self, d):
        self.d = d

    def to_dict(self):
        return self.d

    @property
    def fields(self):
        return list(self.d.keys())

    @staticmethod
    def from_dict(d):
        return TableRow(d)

    def __eq__(self, other):
        if isinstance(other, TableRow):
            return self.d == other.d
        return False

    def __getattr__(self, key):
        if key in self.fields:
            return self.d[key]
        raise AttributeError


class Table:
    def __init__(self, d_list):
        self.d_list = d_list

    @staticmethod
    def from_dict(d_list):
        return Table(d_list)

    def to_dict_list(self):
        return self.d_list

    def __len__(self):
        return len(self.d_list)

    def filter(self, func_filter):
        return Table([d for d in self.d_list if func_filter(d)])

    def map(self, func_map):
        return Table([func_map(d) for d in self.d_list])

    def __eq__(self, other):
        if isinstance(other, Table):
            return self.d_list == other.d_list
        return False

    @property
    def first_d(self):
        return self.d_list[0]

    @property
    def fields(self):
        return list(self.first_d.keys())

    def __getitem__(self, idx):
        return TableRow(self.d_list[idx])

    def __add__(self, other):
        assert isinstance(other, Table)
        assert self.fields == other.fields
        return Table(self.d_list + other.d_list)



"""Test."""
import random
import unittest

from utils.Color import Color


class TestColorX(unittest.TestCase):
    """Test."""

    def test_8bit(self):
        random.seed(0)
        self.assertEqual(Color._8bit(), 197)

    def test_360(self):
        random.seed(0)
        self.assertAlmostEqual(Color._360(), 197)

    def test_float(self):
        random.seed(0)
        self.assertAlmostEqual(Color._float(), 0.8444218515250481)

    def test_percent(self):
        random.seed(0)
        self.assertAlmostEqual(Color._percent(), 49)

    def test_rgb(self):
        random.seed(0)
        self.assertEqual(Color.rgba(), 'rgba(197,215,20,0.26)')

    def test_hsla(self):
        random.seed(0)
        self.assertEqual(Color.hsla(), 'hsla(197,97%,53%,0.04)')



import json

from utils.file.File import File


class JSONFile(File):
    def read(self):
        content = File.read(self)
        return json.loads(content)

    def write(self, data):
        content = json.dumps(data, indent=2)
        File.write(self, content)



"""Utils."""

import logging

from py_console import console, textColor

LEVEL_TO_COLOR = {
    logging.CRITICAL: textColor.MAGENTA,
    logging.ERROR: textColor.RED,
    logging.WARNING: textColor.YELLOW,
    logging.INFO: textColor.BLUE,
    logging.DEBUG: textColor.GREEN,
    logging.NOTSET: textColor.WHITE,
}


class CustomLoggingFormatter(logging.Formatter):
    def format(self, record):
        color = LEVEL_TO_COLOR[record.levelno]
        return console.highlight(
            f'({record.name}-{record.levelname}): {record.msg}', color
        )


class Log(logging.Logger):
    def __init__(self, name: str, level: int = logging.DEBUG):
        super(Log, self).__init__(name, level)
        self.propagate = False

        formatter = CustomLoggingFormatter()
        sh = logging.StreamHandler()
        sh.setLevel(logging.DEBUG)
        sh.setFormatter(formatter)
        self.handlers = [sh]



from utils.Browser import Browser
from utils.Color import Color
from utils.Dict import Dict
from utils.file.CSVFile import CSVFile
from utils.file.Directory import Directory
from utils.file.File import File
from utils.file.FiledVariable import FiledVariable
from utils.file.JSONFile import JSONFile
from utils.file.PDFFile import PDFFile
from utils.file.TSVFile import TSVFile
from utils.file.XSVFile import XSVFile
from utils.file.Zip import Zip
from utils.Git import Git
from utils.List import List
from utils.Log import Log
from utils.String import String
from utils.Table import Table, TableRow
from utils.time import (DAYS_IN, SECONDS_IN, TIME_FORMAT_DATE,
                        TIME_FORMAT_DATE_ID, TIME_FORMAT_TIME,
                        TIME_FORMAT_TIME_ID, TIMEZONE_OFFSET, Time, TimeDelta,
                        TimeFormat, TimeUnit, get_date_id, get_time_id)
from utils.twitter.Tweet import Tweet
from utils.twitter.Twitter import Twitter
from utils.www import WWW



import os
import tempfile
import unittest

from utils.Git import Git

TEST_REPO_URL = 'https://github.com/nuuuwan/utils'
TEST_DIR_REPO = tempfile.TemporaryDirectory().name
TEST_BRACH_NAME = 'main'

TEST_GIT = Git(TEST_REPO_URL)


class TestGit(unittest.TestCase):
    def test_clone(self):
        git = TEST_GIT
        git = Git(TEST_REPO_URL)
        git.clone(TEST_DIR_REPO, force=True)

    @unittest.skip('Fails on windows')
    def test_checkout(self):
        git = TEST_GIT
        git.clone(TEST_DIR_REPO, force=True)
        git.checkout(TEST_BRACH_NAME)
        self.assertTrue(os.path.exists(TEST_DIR_REPO))

        git.clone(TEST_DIR_REPO, force=False)
        git.checkout(TEST_BRACH_NAME)

    def test_add_and_commit(self):
        git = TEST_GIT
        git.clone(TEST_DIR_REPO, force=True)
        git.checkout(TEST_BRACH_NAME)
        cmd = f'echo "test" > {TEST_DIR_REPO}/test.txt'
        os.system(cmd)
        git.add_and_commit('test')

    @unittest.skip('Changes repo code')
    def test_push(self):
        git = TEST_GIT
        git.clone(TEST_DIR_REPO, force=True)
        git.checkout(TEST_BRACH_NAME)
        cmd = f'echo "$(date +%Y%m%d-%H%M%S)" > {TEST_DIR_REPO}/test.txt'
        os.system(cmd)
        git.add_and_commit('test')
        git.push()


if __name__ == '__main__':
    unittest.main()



from utils import Log

log = Log('Tweet')


class Tweet:
    def __init__(
        self, text, image_file_path_list=None, in_reply_to_status_id=None
    ):
        self.text = text
        self.image_file_path_list = image_file_path_list or []
        self.in_reply_to_status_id = in_reply_to_status_id
